#property indicator_separate_window
#property indicator_buffers  4
#property indicator_color1   Tomato
#property indicator_color2   YellowGreen
#property indicator_color3   Green
#property indicator_color4   Red
#property indicator_width1   4
#property indicator_width2   4
#property indicator_width3   4
#property indicator_width4   4
#property indicator_minimum -1
#property indicator_maximum  1

//
//
//
//
//

extern string TimeFrame          = "Current time frame";
extern int    MACD_Fast          =  8;
extern int    MACD_Slow          = 17;
extern int    MACD_MA            =  9;
extern int    MACD_Price         = PRICE_CLOSE;
extern bool   arrowsVisible      = false;
extern string arrowsIdentifier   = "macd bars";
extern double arrowsDisplacement = 0.5;
extern color  arrowsUpColor      = YellowGreen;
extern color  arrowsDnColor      = Red;
extern int    arrowsUpCode       = 233;
extern int    arrowsDnCode       = 234;
extern int    arrowsUpSize       = 1;
extern int    arrowsDnSize       = 1;
//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
double cross[];


string indicatorFileName;
bool   returnBars;
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,ExtMapBuffer1);  SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,ExtMapBuffer2);  SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,ExtMapBuffer3);  SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,ExtMapBuffer4);  SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,cross);

   //
   //
   //
   //
   //
       
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
          
   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+" MACD ("+MACD_Fast+","+MACD_Slow+","+MACD_MA+")");  
   return(0);
}
int deinit() 
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i); if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return (0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { ExtMapBuffer1[0] = limit+1; return(0); }
           if (timeFrame!=Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   
   //
   //
   //
   //
   //
   
   for (int i = limit; i >=0; i--)
   {
      ExtMapBuffer1[i]=EMPTY_VALUE;
      ExtMapBuffer2[i]=EMPTY_VALUE;
      ExtMapBuffer3[i]=EMPTY_VALUE;
      ExtMapBuffer4[i]=EMPTY_VALUE;
   
      int y = iBarShift(NULL,timeFrame,Time[i]);
      double MACD_Main   = iMACD(NULL,timeFrame,MACD_Fast,MACD_Slow,MACD_MA,MACD_Price,MODE_MAIN  ,y);
      double MACD_Signal = iMACD(NULL,timeFrame,MACD_Fast,MACD_Slow,MACD_MA,MACD_Price,MODE_SIGNAL,y);
   
         if(MACD_Signal < MACD_Main)                  ExtMapBuffer2[i] = -1; //Sig Below MACD 
         if(MACD_Signal > MACD_Main)                  ExtMapBuffer1[i] =  1; //Sig above MACD
         if(MACD_Signal < MACD_Main && MACD_Main > 0) ExtMapBuffer3[i] =  1; //Sig below MACD & Above 0
         if(MACD_Signal > MACD_Main && MACD_Main < 0) ExtMapBuffer4[i] = -1; //Sig above MACD & Below 0
         cross[i] = cross[i+1];
            if (MACD_Main > 0) cross[i] =  1;
            if (MACD_Main < 0) cross[i] = -1;
            manageArrow(i);
   }
   return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsVisible)
   {
      ObjectDelete(arrowsIdentifier+":"+Time[i]);
      
      if (cross[i]!=cross[i+1])
      {
         if (cross[i] ==  1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false);
         if (cross[i] == -1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize,true);
      }         
   }      
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,int theSize, bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,   theColor);
         ObjectSet(name,OBJPROP_WIDTH,    theSize);      
         
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsDisplacement * gap);
         else  ObjectSet(name,OBJPROP_PRICE1, Low[i] - arrowsDisplacement * gap);
}